/* Process model C form file: UniApp.pr.c */
/* Portions of this file Copyright (C) MIL 3, Inc. 1992 */



/* OPNET system definitions */
#include <opnet.h>
#include "UniApp.pr.h"
FSM_EXT_DECS



/* Header block */
#define	PK_ARR		op_intrpt_type () == OPC_INTRPT_STRM
#define	SELF_INT1	op_intrpt_type () == OPC_INTRPT_SELF  && op_intrpt_code () == 0
#define END_SIM		op_intrpt_type () ==  OPC_INTRPT_ENDSIM

#define DATA_REC_STAT   0
#define DATA_REC_STAT1  1

#define DATA_SEN_STAT   2
#define DATA_SEN_STAT1  3

#define Meas_Intvl		1

#define MAX_NO_NETS		24 
#define MAX_NO_NODES	1
#define MAX_INIT_TIME	20

#define UDP_PROTOCOL_ID	20

#define ON				1
#define OFF				0

#define VL				if(OFF)
#define UADL			if(OFF)

typedef struct {
	int NetNum;
	int NodeNum;
}IPAddress;

typedef struct {
	char Message[500];
}App_Data;



/* State variable definitions */
typedef struct
	{
	FSM_SYS_STATE
	Objid	                  		sv_my_id;
	Objid	                  		sv_ip_objid;
	double	                 		sv_data_rec;
	double	                 		sv_data_send;
	double	                 		sv_data_iat;
	double	                 		sv_print_time;
	Distribution*           		sv_net_dist;
	Distribution*           		sv_node_dist;
	IPAddress               		sv_LocalAddress;
	} UniApp_state;

#define pr_state_ptr            		((UniApp_state*) SimI_Mod_State_Ptr)
#define my_id                   		pr_state_ptr->sv_my_id
#define ip_objid                		pr_state_ptr->sv_ip_objid
#define data_rec                		pr_state_ptr->sv_data_rec
#define data_send               		pr_state_ptr->sv_data_send
#define data_iat                		pr_state_ptr->sv_data_iat
#define print_time              		pr_state_ptr->sv_print_time
#define net_dist                		pr_state_ptr->sv_net_dist
#define node_dist               		pr_state_ptr->sv_node_dist
#define LocalAddress            		pr_state_ptr->sv_LocalAddress




/* Process model interrupt handling procedure */



void
UniApp ()
	{
	double thru, thru1, tmpDataRec, tmpDataSend, tmp_sim_time, thru2, thru3;
	Packet*  pkptr;
	int in_strm,da1,da2,da3;
	FILE*  fil;
	


	FSM_ENTER (UniApp)

	FSM_BLOCK_SWITCH
		{
		/*---------------------------------------------------------*/
		/** state (init) enter executives **/
		FSM_STATE_ENTER_FORCED (0, state0_enter_exec, "init")
			{
			
			my_id = op_id_self();
			
			ip_objid = op_topo_assoc (my_id, OPC_TOPO_ASSOC_OUT, OPC_OBJMTYPE_MODULE, 0);
			 
			/* Use the object id to get the address */
			op_ima_obj_attr_get (ip_objid, "net_num0", &LocalAddress.NetNum );
			op_ima_obj_attr_get (ip_objid, "node_num0", &LocalAddress.NodeNum );
			
			data_rec = 0;
			data_send = 0;
			print_time = 0;
			
			op_ima_obj_attr_get(my_id,"DATA_IAT", &data_iat);
			
			/*time when he joins initially*/
			net_dist = op_dist_load("uniform_int", 1, MAX_NO_NETS );
			node_dist = op_dist_load("uniform_int", 1, MAX_NO_NODES );
			
			op_intrpt_schedule_self((op_sim_time() + data_iat + MAX_INIT_TIME),0);
			}


		/** state (init) exit executives **/
		FSM_STATE_EXIT_FORCED (0, state0_exit_exec, "init")
			{
			}


		/** state (init) transition processing **/
		FSM_TRANSIT_FORCE (1, state1_enter_exec, ;)
		/*---------------------------------------------------------*/



		/** state (idle) enter executives **/
		FSM_STATE_ENTER_UNFORCED (1, state1_enter_exec, "idle")
			{
			}


		/** blocking after enter executives of unforced state. **/
		FSM_EXIT (3,UniApp)


		/** state (idle) exit executives **/
		FSM_STATE_EXIT_UNFORCED (1, state1_exit_exec, "idle")
			{
			}


		/** state (idle) transition processing **/
		FSM_INIT_COND (PK_ARR)
		FSM_TEST_COND (SELF_INT1)
		FSM_TEST_COND (END_SIM)
		FSM_TEST_LOGIC ("idle")

		FSM_TRANSIT_SWITCH
			{
			FSM_CASE_TRANSIT (0, 2, state2_enter_exec, ;)
			FSM_CASE_TRANSIT (1, 3, state3_enter_exec, ;)
			FSM_CASE_TRANSIT (2, 4, state4_enter_exec, ;)
			}
		/*---------------------------------------------------------*/



		/** state (pk_arr) enter executives **/
		FSM_STATE_ENTER_FORCED (2, state2_enter_exec, "pk_arr")
			{
			
			/*
			** This is a port for all the traffic that was destinated for this host. The 
			** state upon  receiving the packet updates the counter which keeps track of 
			** the number of packets received by this host. Right now it also prints the 
			** number of packets  received for each 20 second  interval. It destroys the 
			** packets that are received.
			*/
			
			pkptr  = op_pk_get (op_intrpt_strm ());
			
			VL	UADL
			{
				App_Data DataRecv;
				int Src_Net, Src_Node;
				Packet* RecivedAppPkptr;
			
				op_pk_nfd_get(pkptr, "src_net", &Src_Net );
				op_pk_nfd_get(pkptr, "src_node", &Src_Node );
				op_pk_nfd_get(pkptr, "data", &RecivedAppPkptr );  
				op_pk_nfd_get(RecivedAppPkptr, "UniData", &DataRecv ); 
			
				printf("(%d.%d) Recieved data message from %d.%d at the time %f\n",
						 LocalAddress.NetNum, LocalAddress.NodeNum, Src_Net, Src_Node, op_sim_time() );
				printf(" %s \n", &DataRecv.Message );    
			
				op_pk_destroy (RecivedAppPkptr);
			}
			
			/*
			** Probes to measure the amount data received by the UDP port
			*/
			data_rec++;
			thru = data_rec;
			op_stat_local_write( DATA_REC_STAT, thru );
			tmpDataRec++;
			if(op_sim_time() > tmp_sim_time + Meas_Intvl )
			{
				thru1 = tmpDataRec/Meas_Intvl;
				op_stat_local_write( DATA_REC_STAT1, thru1);
				tmpDataRec = 0;
				tmp_sim_time = tmp_sim_time + Meas_Intvl;
			}
			
			op_pk_destroy (pkptr);
			}


		/** state (pk_arr) exit executives **/
		FSM_STATE_EXIT_FORCED (2, state2_exit_exec, "pk_arr")
			{
			}


		/** state (pk_arr) transition processing **/
		FSM_TRANSIT_FORCE (1, state1_enter_exec, ;)
		/*---------------------------------------------------------*/



		/** state (data) enter executives **/
		FSM_STATE_ENTER_FORCED (3, state3_enter_exec, "data")
			{
				App_Data DataSent;
				Packet* ip_pkptr;
				int dest_net, dest_node, data_len;
			
				dest_net = op_dist_outcome( net_dist );
				dest_node = op_dist_outcome( node_dist );
			
				sprintf( DataSent.Message, "This was data sending to the host %d.%d by %d.%d \n", 
						 dest_net, dest_node, LocalAddress.NetNum, LocalAddress.NodeNum );
				pkptr = op_pk_create_fmt("Unicast_Appl");
			
				op_pk_nfd_set(pkptr, "UniData", &DataSent, op_prg_mem_alloc,
							  op_prg_mem_free, sizeof(App_Data) );
			
				data_len = op_pk_total_size_get( pkptr )/8;
			
				ip_pkptr = op_pk_create_fmt("ip_dgram");
				op_pk_nfd_set(ip_pkptr, "data", pkptr );
				op_pk_nfd_set(ip_pkptr, "prot_id", UDP_PROTOCOL_ID );
				op_pk_bulk_size_set(ip_pkptr, data_len*8 );
			
				op_pk_nfd_set(ip_pkptr, "dest_net", dest_net );
				op_pk_nfd_set(ip_pkptr, "dest_node", dest_node );
				op_pk_nfd_set(ip_pkptr, "src_net", LocalAddress.NetNum );
				op_pk_nfd_set(ip_pkptr, "src_node", LocalAddress.NodeNum );
			
				op_pk_nfd_set(ip_pkptr, "orig_len", data_len );
				op_pk_nfd_set(ip_pkptr, "frag_len", data_len );
				op_pk_nfd_set(ip_pkptr, "ttl", 2 );
				op_pk_nfd_set(ip_pkptr, "frag", 0 );
			
				VL	if(OFF)	printf("Sending a data pk to %d.%d. PACKET ID = %d\n", 
									dest_net, dest_node, op_pk_id(ip_pkptr) );  
				op_pk_send(ip_pkptr,0);
				op_intrpt_schedule_self((op_sim_time() + (data_iat)),0);
			
				if(OFF)
				{
					data_send++;
					tmpDataSend++;
					thru2 = data_send;
					op_stat_local_write( DATA_SEN_STAT, thru2 );
					
					if(op_sim_time() > tmp_sim_time + Meas_Intvl )
					{
						thru3 = tmpDataSend/Meas_Intvl;
						op_stat_local_write( DATA_SEN_STAT1, thru3);
						tmpDataSend = 0;
						tmp_sim_time = tmp_sim_time + Meas_Intvl;
					}
				}
			
			}


		/** state (data) exit executives **/
		FSM_STATE_EXIT_FORCED (3, state3_exit_exec, "data")
			{
			}


		/** state (data) transition processing **/
		FSM_TRANSIT_FORCE (1, state1_enter_exec, ;)
		/*---------------------------------------------------------*/



		/** state (END) enter executives **/
		FSM_STATE_ENTER_UNFORCED (4, state4_enter_exec, "END")
			{
			fil = fopen("OP","a+");
			fprintf(fil,"\nAve DATA recieved : %f pks/sec",data_rec/op_sim_time());
			fclose(fil);
			
			printf("(%d.%d) Number of packets received by me is %f \n", 
					LocalAddress.NetNum, LocalAddress.NodeNum, data_rec );
			printf("(%d.%d) Number of packets sended by me is %f \n", 
					LocalAddress.NetNum, LocalAddress.NodeNum, data_send );
			}


		/** blocking after enter executives of unforced state. **/
		FSM_EXIT (9,UniApp)


		/** state (END) exit executives **/
		FSM_STATE_EXIT_UNFORCED (4, state4_exit_exec, "END")
			{
			}


		/** state (END) transition processing **/
		FSM_TRANSIT_MISSING ("END")
		/*---------------------------------------------------------*/



		}


	FSM_EXIT (0,UniApp)
	}




void
UniApp_svar (prs_ptr,var_name,var_p_ptr)
	UniApp_state		*prs_ptr;
	char			*var_name, **var_p_ptr;
	{

	FIN (UniApp_svar (prs_ptr))

	*var_p_ptr = VOS_NIL;
	if (Vos_String_Equal ("my_id" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_my_id);
	if (Vos_String_Equal ("ip_objid" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_ip_objid);
	if (Vos_String_Equal ("data_rec" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_data_rec);
	if (Vos_String_Equal ("data_send" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_data_send);
	if (Vos_String_Equal ("data_iat" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_data_iat);
	if (Vos_String_Equal ("print_time" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_print_time);
	if (Vos_String_Equal ("net_dist" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_net_dist);
	if (Vos_String_Equal ("node_dist" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_node_dist);
	if (Vos_String_Equal ("LocalAddress" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_LocalAddress);

	FOUT;
	}




void
UniApp_diag ()
	{
	double thru, thru1, tmpDataRec, tmpDataSend, tmp_sim_time, thru2, thru3;
	Packet*  pkptr;
	int in_strm,da1,da2,da3;
	FILE*  fil;
	

	FIN (UniApp_diag ())


	FOUT;
	}




void
UniApp_terminate ()
	{
	double thru, thru1, tmpDataRec, tmpDataSend, tmp_sim_time, thru2, thru3;
	Packet*  pkptr;
	int in_strm,da1,da2,da3;
	FILE*  fil;
	

	FIN (UniApp_terminate ())


	FOUT;
	}



Compcode
UniApp_init (pr_state_pptr)
	UniApp_state			**pr_state_pptr;
	{
	static VosT_Cm_Obtype	obtype = OPC_NIL;

	FIN (UniApp_init (pr_state_pptr))

	if (obtype == OPC_NIL)
		{
		if (Vos_Catmem_Register ("proc state vars (UniApp)",
			sizeof (UniApp_state), Vos_Nop, &obtype) == VOSC_FAILURE)
			FRET (OPC_COMPCODE_FAILURE)
		}

	if ((*pr_state_pptr = (UniApp_state*) Vos_Catmem_Alloc (obtype, 1)) == OPC_NIL)
		FRET (OPC_COMPCODE_FAILURE)
	else
		{
		(*pr_state_pptr)->current_block = 0;
		FRET (OPC_COMPCODE_SUCCESS)
		}
	}



